/*
 *  nautilus-search-tool.c
 * 
 *  Copyright (C) 2005-2009 Andrea Zagli <azagli@libero.it>
 *
 *  This file is part of nautilus-search-tool.
 *  
 *  nautilus-search-tool is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  nautilus-search-tool is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with nautilus-search-tool; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

#ifdef HAVE_CONFIG_H
	#include <config.h> /* for GETTEXT_PACKAGE */
#endif

#include <ctype.h>
#include <stdlib.h>

#include "nautilus-search-tool.h"

#include <nautilus-extension.h>

#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>

static void nautilus_search_tool_instance_init (NautilusSearchTool *cvs);
static void nautilus_search_tool_class_init    (NautilusSearchToolClass *class);

static GType search_tool_type = 0;

static void
search_tool_callback (NautilusMenuItem *item,
                      NautilusFileInfo *file_info)
{
	gchar *uri,
	      *argv[3],
	      *working_directory,
	      *quoted_directory;

	g_print ("Search Tool selected\n");

	uri = nautilus_file_info_get_uri (file_info);

	if (uri)
		{
			if (strcmp (nautilus_file_info_get_uri_scheme (file_info), "x-nautilus-desktop") == 0)
				{
					working_directory = g_strdup_printf ("%s/Desktop", g_get_home_dir ());
				}
			else
				{
					working_directory = g_filename_from_uri (uri, NULL, NULL);
				}
		}
	else
		{
			working_directory = g_strdup (g_get_home_dir ());
		}

	if (!nautilus_file_info_is_directory (file_info))
		working_directory = g_path_get_dirname (working_directory);

	g_free (uri);

	argv[0] = g_strdup ("gnome-search-tool");
	quoted_directory = g_shell_quote (working_directory);

#ifdef GNOME_SEARCH_TOOL_VERSION
	guint i;
	gchar *version;

	gchar **array_version = g_strsplit (GNOME_SEARCH_TOOL_VERSION, " ", -1);
	guint array_length = g_strv_length (array_version);

	version = (gchar *) g_malloc (array_length * sizeof (gchar));
	for (i = 0; i < array_length; i++)
		{
			if (isdigit (array_version[i][0]))
				{
					version = g_strdup (array_version[i]);
					g_strfreev (array_version);

					array_version = g_strsplit (version, ".", -1);
					break;
				}
		}

	if (atoi (array_version[0]) > 2 || (atoi (array_version[0]) == 2 && atoi (array_version[1]) >= 22))
		{
			argv[1] = g_strdup_printf ("--path=%s", quoted_directory);
			argv[2] = NULL;
		}
	else
		{
			argv[1] = NULL;
		}
		

	g_free (version);
	g_strfreev (array_version);
#else
	argv[1] = NULL;
#endif /* GNOME_SEARCH_TOOL */

	g_spawn_async (working_directory,
	               argv,
	               NULL,
	               G_SPAWN_SEARCH_PATH,
	               NULL,
	               NULL,
	               NULL,
	               NULL);

	g_free (quoted_directory);
	g_free (working_directory);
	g_free (argv[0]);
	g_free (argv[1]);
}

static NautilusMenuItem *
search_tool_menu_item_new ()
{
	return nautilus_menu_item_new ("NautilusSearchTool::search_tool",
	                               _("Search for Files..."),
	                               _("Find files, folders, and documents on selected position"),
	                               "gnome-searchtool");
}

static GList *
nautilus_search_tool_get_background_items (NautilusMenuProvider *provider,
                                           GtkWidget            *window,
                                           NautilusFileInfo     *file_info)
{
	NautilusMenuItem *item;

	item = search_tool_menu_item_new ();
	g_signal_connect (item, "activate",
	                  G_CALLBACK (search_tool_callback),
	                  file_info);

	return g_list_append (NULL, item);
}

GList *
nautilus_search_tool_get_file_items (NautilusMenuProvider *provider,
                                     GtkWidget            *window,
                                     GList                *files)
{
	NautilusMenuItem *item;

	if (g_list_length (files) != 1)
		return NULL;

	item = search_tool_menu_item_new ();
	g_signal_connect (item, "activate",
	                  G_CALLBACK (search_tool_callback),
	                  files->data);

	return g_list_append (NULL, item);
}

static void
nautilus_search_tool_menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_background_items = nautilus_search_tool_get_background_items;
	iface->get_file_items = nautilus_search_tool_get_file_items;
}

static void 
nautilus_search_tool_instance_init (NautilusSearchTool *cvs)
{
}

static void
nautilus_search_tool_class_init (NautilusSearchToolClass *class)
{
}

GType
nautilus_search_tool_get_type (void) 
{
	return search_tool_type;
}

void
nautilus_search_tool_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
		sizeof (NautilusSearchToolClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) nautilus_search_tool_class_init,
		NULL, 
		NULL,
		sizeof (NautilusSearchTool),
		0,
		(GInstanceInitFunc) nautilus_search_tool_instance_init,
	};

	static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_search_tool_menu_provider_iface_init,
		NULL,
		NULL
	};

	search_tool_type = g_type_module_register_type (module,
	                                                G_TYPE_OBJECT,
	                                                "NautilusSearchTool",
	                                                &info, 0);

	g_type_module_add_interface (module,
	                             search_tool_type,
	                             NAUTILUS_TYPE_MENU_PROVIDER,
	                             &menu_provider_iface_info);
}
